@@ -53,21 +53,26 @@ from ...constants import (
 from ...models import (
     Abstract, AccessRestrict, Accruals, AcqInfo, AgencyName, AltFormAvail,
     Appraisal, Arrangement, Author, Bibliography, BiogHist, Container,
-    ControlAccess, ControlNote, ConventionDeclaration, CorpName, CorpNamePart,
-    CustodHist, DAOSet, DAOSetDAO, DIdDAO, DIdNote, DIdPhysDescStructured,
+    ControlAccess, ControlNote, ConventionDeclaration, CustodHist, DAOSet,
+    DAOSetDAO, DIdDAO, DIdNote, DIdPhysDescStructured,
     DIdPhysDescStructuredDimensions, DIdPhysDescStructuredPhysFacet, EAD,
-    EventDescription, FamName, FamNamePart, FilePlan, Index, LangMaterial,
-    LangMaterialLanguage, LanguageDeclaration, LanguageSet,
-    LanguageSetLanguage, LanguageSetScript, LegalStatus, LocalControl,
-    LocalTypeDeclaration, MaintenanceEvent, MaterialSpec, Name, NamePart, ODD,
-    OriginalsLoc, Origination, OtherAgencyCode, OtherFindAid, OtherRecordID,
-    PersName, PersNamePart, PhysDesc, PhysDescSet,
-    PhysDescSetPhysDescStructured, PhysDescSetPhyDescStructuredDimensions,
+    EventDescription, FilePlan, Index, LangMaterial, LangMaterialLanguage,
+    LanguageDeclaration, LanguageSet, LanguageSetLanguage, LanguageSetScript,
+    LegalStatus, LocalControl, LocalTypeDeclaration, MaintenanceEvent,
+    MaterialSpec, ODD, OriginalsLoc, Origination, OriginationCorpName,
+    OriginationCorpNamePart, OriginationFamName, OriginationFamNamePart,
+    OriginationName, OriginationNamePart, OriginationPersName,
+    OriginationPersNamePart, OtherAgencyCode, OtherFindAid, OtherRecordID,
+    PhysDesc, PhysDescSet, PhysDescSetPhysDescStructured,
+    PhysDescSetPhyDescStructuredDimensions,
     PhysDescSetPhysDescStructuredPhysFacet, PhysLoc, PhysTech, PreferCite,
     ProcessInfo, RelatedMaterial, Relation, RelationDateRange,
     RelationDateSingle, RelationEntry, Repository, RepositoryAddressLine,
-    Representation, RightsDeclaration, ScopeContent, SeparatedMaterial, Source,
-    SourceEntry, Sponsor, Subtitle, TitleProper, UnitDate, UnitDateStructured,
+    RepositoryCorpName, RepositoryCorpNamePart, RepositoryFamName,
+    RepositoryFamNamePart, RepositoryName, RepositoryNamePart,
+    RepositoryPersName, RepositoryPersNamePart, Representation,
+    RightsDeclaration, ScopeContent, SeparatedMaterial, Source, SourceEntry,
+    Sponsor, Subtitle, TitleProper, UnitDate, UnitDateStructured,
     UnitDateStructuredDateRange, UnitDateStructuredDateSingle, UnitId,
     UnitTitle, UseRestrict)
 
@@ -646,30 +651,22 @@ class Command(BaseCommand):
         language_obj.save()
 
     def _import_languagedeclaration(self, ld, ead_obj):
-        # Cheat by looking for an existing LanguageDeclaration that
-        # shares only language code and script code, ignoring all
-        # other fields.
         language = ld[0]
         script = ld[1]
         langcode = search_term('iso639-2', language.get('langcode'))
         scriptcode = search_term('iso15924', script.get('scriptcode'))
-        try:
-            ld_obj = LanguageDeclaration.objects.get(
-                language_langcode=langcode, script_el_scriptcode=scriptcode)
-        except LanguageDeclaration.DoesNotExist:
-            ld_obj = LanguageDeclaration()
-            import_attrs(ld, LANGUAGEDECLARATION_ATTRS, ld_obj)
-            import_attrs(language, LANGUAGE_ATTRS, ld_obj, 'language_')
-            import_attrs(script, SCRIPT_ATTRS, ld_obj, 'script_el_')
-            ld_obj.language = language.text
-            ld_obj.language_langcode = langcode
-            ld_obj.script_el = script.text
-            ld_obj.script_el_scriptcode = scriptcode
-            for descriptivenote in xpath(ld, 'e:descriptivenote'):
-                self._import_descriptivenote(descriptivenote, ld_obj)
-            ld_obj.full_clean()
-            ld_obj.save()
-        ead_obj.languagedeclarations.add(ld_obj)
+        ld_obj = LanguageDeclaration(control=ead_obj)
+        import_attrs(ld, LANGUAGEDECLARATION_ATTRS, ld_obj)
+        import_attrs(language, LANGUAGE_ATTRS, ld_obj, 'language_')
+        import_attrs(script, SCRIPT_ATTRS, ld_obj, 'script_el_')
+        ld_obj.language = language.text
+        ld_obj.language_langcode = langcode
+        ld_obj.script_el = script.text
+        ld_obj.script_el_scriptcode = scriptcode
+        for descriptivenote in xpath(ld, 'e:descriptivenote'):
+            self._import_descriptivenote(descriptivenote, ld_obj)
+        ld_obj.full_clean()
+        ld_obj.save()
 
     def _import_languageset(self, languageset, langmaterial_obj):
         languageset_obj = LanguageSet(langmaterial=langmaterial_obj)
@@ -772,24 +769,21 @@ class Command(BaseCommand):
         materialspec_obj.full_clean()
         materialspec_obj.save()
 
-    def _import_name(self, name, name_model, part_model):
+    def _import_name(self, name, name_model, part_model, parent_obj, field):
         """Import name from `name` element, using `name_model` and
-        `part_model`, and return it. Return an existing name if one
-        matches."""
+        `part_model`, and return it."""
         assembled_name = ''.join(xpath(name, 'e:part//text()'))
         if not assembled_name:
             raise CommandError(EMPTY_NAME_MSG.format(self._path))
-        try:
-            name_obj = name_model.objects.get(assembled_name=assembled_name)
-        except name_model.DoesNotExist:
-            name_obj = name_model(assembled_name=assembled_name)
-            import_attrs(name, NAME_ATTRS, name_obj)
-            name_obj.full_clean()
-            name_obj.save()
-            for idx, part in enumerate(name):
-                self._import_part(part, name_obj, part_model, idx)
-            name_obj.full_clean()
-            name_obj.save()  # To set the assembled_name from the parts.
+        name_obj = name_model()
+        setattr(name_obj, field, parent_obj)
+        import_attrs(name, NAME_ATTRS, name_obj)
+        name_obj.full_clean()
+        name_obj.save()
+        for idx, part in enumerate(name):
+            self._import_part(part, name_obj, part_model, idx)
+        name_obj.full_clean()
+        name_obj.save()  # To set the assembled_name from the parts.
         return name_obj
 
     def _import_notestmt(self, notestmt, ead_obj):
@@ -821,17 +815,20 @@ class Command(BaseCommand):
         origination_obj.full_clean()
         origination_obj.save()
         for corpname in xpath(origination, 'e:corpname'):
-            name_obj = self._import_name(corpname, CorpName, CorpNamePart)
-            origination_obj.corpnames.add(name_obj)
+            self._import_name(
+                corpname, OriginationCorpName, OriginationCorpNamePart,
+                origination_obj, 'origination')
         for famname in xpath(origination, 'e:famname'):
-            name_obj = self._import_name(famname, FamName, FamNamePart)
-            origination_obj.famnames.add(name_obj)
+            self._import_name(
+                famname, OriginationFamName, OriginationFamNamePart,
+                origination_obj, 'origination')
         for name in xpath(origination, 'e:name'):
-            name_obj = self._import_name(name, Name, NamePart)
-            origination_obj.names.add(name_obj)
+            self._import_name(name, OriginationName, OriginationNamePart,
+                              origination_obj, 'origination')
         for persname in xpath(origination, 'e:persname'):
-            name_obj = self._import_name(persname, PersName, PersNamePart)
-            origination_obj.persnames.add(name_obj)
+            self._import_name(
+                persname, OriginationPersName, OriginationPersNamePart,
+                origination_obj, 'origination')
         origination_obj.full_clean()
 
     def _import_otheragencycode(self, otheragencycode, ead_obj):
@@ -1002,31 +999,25 @@ class Command(BaseCommand):
         relationentry_obj.save()
 
     def _import_repository(self, repository, ead_obj):
-        assembled_name = ''.join(xpath(repository, 'e:*/e:part//text()'))
-        try:
-            repository_obj = Repository.objects.get(
-                assembled_name=assembled_name)
-            ead_obj.repositories.add(repository_obj)
-            return
-        except Repository.DoesNotExist:
-            pass
-        repository_obj = Repository()
+        repository_obj = Repository(did=ead_obj)
         import_attrs(repository, REPOSITORY_ATTRS, repository_obj)
         repository_obj.full_clean()
         repository_obj.save()
-        ead_obj.repositories.add(repository_obj)
         for corpname in xpath(repository, 'e:corpname'):
-            name_obj = self._import_name(corpname, CorpName, CorpNamePart)
-            repository_obj.corpnames.add(name_obj)
+            self._import_name(
+                corpname, RepositoryCorpName, RepositoryCorpNamePart,
+                repository_obj, 'repository')
         for famname in xpath(repository, 'e:famname'):
-            name_obj = self._import_name(famname, FamName, FamNamePart)
-            repository_obj.famnames.add(name_obj)
+            self._import_name(
+                famname, RepositoryFamName, RepositoryFamNamePart,
+                repository_obj, 'repository')
         for name in xpath(repository, 'e:name'):
-            name_obj = self._import_name(name, Name, NamePart)
-            repository_obj.names.add(name_obj)
+            self._import_name(name, RepositoryName, RepositoryNamePart,
+                              repository_obj, 'repository')
         for persname in xpath(repository, 'e:persname'):
-            name_obj = self._import_name(persname, PersName, PersNamePart)
-            repository_obj.persnames.add(name_obj)
+            self._import_name(
+                persname, RepositoryPersName, RepositoryPersNamePart,
+                repository_obj, 'repository')
         for address in xpath(repository, 'e:address'):
             self._import_address(address, repository_obj)
         repository_obj.full_clean()
