@@ -486,7 +486,8 @@ class LanguageMixin(models.Model):
     language = models.CharField(max_length=32)
 
 
-class NameMixin(models.Model):
+class NameMixin(AccessMixin, CommonMixin, EncodingAnalogMixin,
+                LangScriptMixin, LocalTypeMixin, NormalMixin, RelatorMixin):
     """Provides an extra field and save method for name models that stores
     a name assembled from its parts."""
     assembled_name = models.TextField(blank=True)
@@ -988,12 +989,13 @@ class ArchDescSection(DIdSection, RelationsSection):
     'author_set', 'bibliography_set', 'bioghist_set', 'container_set',
     'controlaccess_set', 'controlnote_set', 'conventiondeclaration_set',
     'custodhist_set', 'dao_set', 'daoset_set', 'didnote_set', 'fileplan_set',
-    'index_set', 'langmaterial_set', 'legalstatus_set', 'localcontrol_set',
-    'localtypedeclaration_set', 'maintenanceevent_set', 'materialspec_set',
-    'odd_set', 'originalsloc_set', 'origination_set', 'otheragencycode_set',
-    'otherfindaid_set', 'otherrecordid_set', 'physdesc_set', 'physdescset_set',
+    'index_set', 'langmaterial_set', 'languagedeclaration_set',
+    'legalstatus_set', 'localcontrol_set', 'localtypedeclaration_set',
+    'maintenanceevent_set', 'materialspec_set', 'odd_set', 'originalsloc_set',
+    'origination_set', 'otheragencycode_set', 'otherfindaid_set',
+    'otherrecordid_set', 'physdesc_set', 'physdescset_set',
     'physdescstructured_set', 'physloc_set', 'phystech_set', 'prefercite_set',
-    'processinfo_set', 'relatedmaterial_set', 'relation_set',
+    'processinfo_set', 'relatedmaterial_set', 'relation_set', 'repository_set',
     'representation_set', 'rightsdeclaration_set', 'scopecontent_set',
     'separatedmaterial_set', 'source_set', 'sponsor_set', 'subtitle_set',
     'titleproper_set', 'unitdate_set', 'unitdatestructured_set', 'unitid_set',
@@ -1008,14 +1010,6 @@ class EAD(BaseMixin, CommonMixin, LangScriptMixin, RelatedEncodingMixin,
     #
     # In order to provide some degree of organisation, sections are in
     # their own abstract models and inherited.
-    #
-    # Some fields that properly belong in one of the *Section abstract
-    # models are defined here in order to avoid having to specify a
-    # cumbersome related_name.
-    languagedeclarations = models.ManyToManyField(
-        'LanguageDeclaration', blank=True)
-    repositories = models.ManyToManyField(
-        'Repository', blank=True, help_text=constants.REPOSITORY_HELP)
 
     def serialise(self):
         """Return this object (and its related objects) serialised as a
@@ -1171,24 +1165,6 @@ class ConventionDeclaration(CommonMixin, EncodingAnalogMixin, LangScriptMixin,
     control = models.ForeignKey(EAD, on_delete=models.CASCADE)
 
 
-@reversion.register(follow=['part_set'])
-class CorpName(AccessMixin, CommonMixin, EncodingAnalogMixin, LangScriptMixin,
-               LocalTypeMixin, NormalMixin, RelatorMixin, NameMixin):
-    """Corresponds to EAD3 corpname."""
-    pass
-
-
-@reversion.register()
-class CorpNamePart(PartMixin):
-    """Corresponds to EAD3 part as a child of corpname."""
-    name = models.ForeignKey(
-        CorpName, on_delete=models.CASCADE, related_name='part_set')
-
-    class Meta:
-        ordering = ['order']
-        unique_together = ['name', 'order']
-
-
 @reversion.register()
 class CustodHist(CommonMixin, EncodingAnalogMixin, LangScriptMixin,
                  LocalTypeMixin):
@@ -1258,23 +1234,8 @@ class EventDescription(CommonMixin, EncodingAnalogMixin, LangScriptMixin,
         'MaintenanceEvent', on_delete=models.CASCADE)
     eventdescription = models.TextField()
 
-
-@reversion.register(follow=['part_set'])
-class FamName(AccessMixin, CommonMixin, EncodingAnalogMixin, LangScriptMixin,
-              LocalTypeMixin, NormalMixin, RelatorMixin, NameMixin):
-    """Corresponds to EAD3 famname."""
-    pass
-
-
-@reversion.register()
-class FamNamePart(PartMixin):
-    """Corresponds to EAD3 part as a child of famname."""
-    name = models.ForeignKey(
-        FamName, on_delete=models.CASCADE, related_name='part_set')
-
-    class Meta:
-        ordering = ['order']
-        unique_together = ['name', 'order']
+    def __str__(self):
+        return 'Event Description: {}'.format(self.eventdescription)
 
 
 @reversion.register()
@@ -1312,13 +1273,8 @@ class LangMaterialLanguage(CommonMixin, EncodingAnalogMixin, LabelMixin,
 @reversion.register()
 class LanguageDeclaration(CommonMixin, EncodingAnalogMixin, LangScriptMixin,
                           DescriptiveNoteMixin, LanguageMixin, ScriptMixin):
-    """Corresponds to EAD3 languagedeclaration.
-
-    Exists in a ManyToMany relationship with EAD (Control), since it
-    seems likely that within a project most records will use the same
-    language(s) and script(s).
-
-    """
+    """Corresponds to EAD3 languagedeclaration."""
+    control = models.ForeignKey(EAD, on_delete=models.CASCADE)
     language_altrender = deepcopy(altrender)
     language_audience = deepcopy(audience)
     language_encodinganalog = deepcopy(encodinganalog)
@@ -1504,24 +1460,6 @@ class MaterialSpec(CommonMixin, EncodingAnalogMixin, LabelMixin,
     materialspec = models.TextField()
 
 
-@reversion.register(follow=['part_set'])
-class Name(AccessMixin, CommonMixin, EncodingAnalogMixin, LangScriptMixin,
-           LocalTypeMixin, NormalMixin, RelatorMixin, NameMixin):
-    """Corresponds to EAD3 name."""
-    pass
-
-
-@reversion.register()
-class NamePart(PartMixin):
-    """Corresponds to EAD3 part as a child of name."""
-    name = models.ForeignKey(
-        Name, on_delete=models.CASCADE, related_name='part_set')
-
-    class Meta:
-        ordering = ['order']
-        unique_together = ['name', 'order']
-
-
 @reversion.register()
 class ODD(CommonMixin, EncodingAnalogMixin, LangScriptMixin,
           LocalTypeMixin):
@@ -1543,25 +1481,97 @@ class Origination(CommonMixin, EncodingAnalogMixin, LabelMixin,
                   LangScriptMixin, LocalTypeMixin):
     """Corresponds to EAD3 origination."""
     did = models.ForeignKey(EAD, on_delete=models.CASCADE)
-    corpnames = models.ManyToManyField(CorpName, blank=True)
-    famnames = models.ManyToManyField(FamName, blank=True)
-    names = models.ManyToManyField(Name, blank=True)
-    persnames = models.ManyToManyField('PersName', blank=True)
 
     def clean(self):
         # There must be at least one CorpName, FamnName, Name, or
-        # PersName associated with an Origination. Since these are
-        # many to many fields, we must be able to create an
-        # Origination instance before populating them, so do not run
-        # this validation if there is no primary key.
+        # PersName associated with an Origination. Since we must be
+        # able to create an Origination instance before populating the
+        # names, do not run this validation if there is no primary
+        # key.
         if self.pk is None:
             return
-        if self.corpnames.count() + self.famnames.count() + \
-                self.names.count() + self.persnames.count() == 0:
+        if self.corpname_set.count() + self.famname_set.count() + \
+                self.name_set.count() + self.persname_set.count() == 0:
             raise ValidationError(
                 'Origination must reference at least one named entity.')
 
 
+@reversion.register(follow=['part_set'])
+class OriginationCorpName(NameMixin):
+    """Corresponds to EAD3 corpname as a child of origination."""
+    origination = models.ForeignKey('Origination', on_delete=models.CASCADE,
+                                    related_name='corpname_set')
+
+
+@reversion.register()
+class OriginationCorpNamePart(PartMixin):
+    """Corresponds to EAD3 part as a child of corpname and grandchild of
+    origination."""
+    name = models.ForeignKey(
+        OriginationCorpName, on_delete=models.CASCADE, related_name='part_set')
+
+    class Meta:
+        ordering = ['order']
+        unique_together = ['name', 'order']
+
+
+@reversion.register(follow=['part_set'])
+class OriginationFamName(NameMixin):
+    """Corresponds to EAD3 famname as a child of origination."""
+    origination = models.ForeignKey(Origination, on_delete=models.CASCADE,
+                                    related_name='famname_set')
+
+
+@reversion.register()
+class OriginationFamNamePart(PartMixin):
+    """Corresponds to EAD3 part as a child of famname and grandchild of
+    origination."""
+    name = models.ForeignKey(
+        OriginationFamName, on_delete=models.CASCADE, related_name='part_set')
+
+    class Meta:
+        ordering = ['order']
+        unique_together = ['name', 'order']
+
+
+@reversion.register(follow=['part_set'])
+class OriginationName(NameMixin):
+    """Corresponds to EAD3 name as a child of origination."""
+    origination = models.ForeignKey(Origination, on_delete=models.CASCADE,
+                                    related_name='name_set')
+
+
+@reversion.register()
+class OriginationNamePart(PartMixin):
+    """Corresponds to EAD3 part as a child of name and grandchild of
+    origination."""
+    name = models.ForeignKey(
+        OriginationName, on_delete=models.CASCADE, related_name='part_set')
+
+    class Meta:
+        ordering = ['order']
+        unique_together = ['name', 'order']
+
+
+@reversion.register(follow=['part_set'])
+class OriginationPersName(NameMixin):
+    """Corresponds to EAD3 persname as a child of origination."""
+    origination = models.ForeignKey(Origination, on_delete=models.CASCADE,
+                                    related_name='persname_set')
+
+
+@reversion.register()
+class OriginationPersNamePart(PartMixin):
+    """Corresponds to EAD3 part as a child of persname and grandchild of
+    origination."""
+    name = models.ForeignKey(
+        OriginationPersName, on_delete=models.CASCADE, related_name='part_set')
+
+    class Meta:
+        ordering = ['order']
+        unique_together = ['name', 'order']
+
+
 @reversion.register()
 class OtherAgencyCode(CommonMixin, EncodingAnalogMixin, LangScriptMixin,
                       LocalTypeMixin):
@@ -1586,24 +1596,6 @@ class OtherRecordID(CommonMixin, EncodingAnalogMixin, LangScriptMixin,
     otherrecordid = models.CharField(max_length=512)
 
 
-@reversion.register(follow=['part_set'])
-class PersName(AccessMixin, CommonMixin, EncodingAnalogMixin, LangScriptMixin,
-               LocalTypeMixin, NormalMixin, RelatorMixin, NameMixin):
-    """Corresponds to EAD3 persname."""
-    pass
-
-
-@reversion.register()
-class PersNamePart(PartMixin):
-    """Corresponds to EAD3 part as a child of persname."""
-    name = models.ForeignKey(
-        PersName, on_delete=models.CASCADE, related_name='part_set')
-
-    class Meta:
-        ordering = ['order']
-        unique_together = ['name', 'order']
-
-
 @reversion.register()
 class PhysDesc(CommonMixin, EncodingAnalogMixin, LabelMixin, LangScriptMixin,
                LocalTypeMixin):
@@ -1749,27 +1741,23 @@ class RelationEntry(CommonMixin, EncodingAnalogMixin, LangScriptMixin,
 class Repository(CommonMixin, EncodingAnalogMixin, LabelMixin, LangScriptMixin,
                  LocalTypeMixin):
     """Corresponds to EAD3 repository."""
+    did = models.ForeignKey(EAD, on_delete=models.CASCADE)
     address_altrender = deepcopy(altrender)
     address_audience = deepcopy(audience)
     address_ead_id = deepcopy(ead_id)
     address_lang = deepcopy(lang)
     address_script = deepcopy(script)
     assembled_name = models.TextField(blank=True)
-    corpnames = models.ManyToManyField(CorpName, blank=True)
-    famnames = models.ManyToManyField(FamName, blank=True)
-    names = models.ManyToManyField(Name, blank=True)
-    persnames = models.ManyToManyField(PersName, blank=True)
 
     def clean(self):
         # There must be at least one CorpName, FamnName, Name, or
-        # PersName associated with a Repository. Since these are
-        # many to many fields, we must be able to create an
-        # Repository instance before populating them, so do not run
-        # this validation if there is no primary key.
+        # PersName associated with a Repository. Since we must be able
+        # to create an Repository instance before populating them, do
+        # not run this validation if there is no primary key.
         if self.pk is None:
             return
-        if self.corpnames.all().count() + self.famnames.all().count() + \
-           self.names.all().count() + self.persnames.all().count() == 0:
+        if self.corpname_set.count() + self.famname_set.count() + \
+           self.name_set.count() + self.persname_set.count() == 0:
             raise ValidationError(
                 'Repository must reference at least one named entity.')
 
@@ -1779,13 +1767,13 @@ class Repository(CommonMixin, EncodingAnalogMixin, LabelMixin, LangScriptMixin,
         if self.pk:
             name_parts = []
             name_parts.extend(
-                self.corpnames.values_list('assembled_name', flat=True))
+                self.corpname_set.values_list('assembled_name', flat=True))
             name_parts.extend(
-                self.famnames.values_list('assembled_name', flat=True))
-            name_parts.extend(self.names.values_list(
+                self.famname_set.values_list('assembled_name', flat=True))
+            name_parts.extend(self.name_set.values_list(
                 'assembled_name', flat=True))
             name_parts.extend(
-                self.persnames.values_list('assembled_name', flat=True))
+                self.persname_set.values_list('assembled_name', flat=True))
             self.assembled_name = '; '.join(name_parts)
         super().save(*args, **kwargs)
 
@@ -1801,6 +1789,81 @@ class RepositoryAddressLine(CommonMixin, LangScriptMixin, LocalTypeMixin):
     addressline = models.TextField()
 
 
+@reversion.register(follow=['part_set'])
+class RepositoryCorpName(NameMixin):
+    """Corresponds to EAD3 corpname as a child of repository."""
+    repository = models.ForeignKey(Repository, on_delete=models.CASCADE,
+                                   related_name='corpname_set')
+
+
+@reversion.register()
+class RepositoryCorpNamePart(PartMixin):
+    """Corresponds to EAD3 part as a child of corpname and grandchild of
+    repository."""
+    name = models.ForeignKey(
+        RepositoryCorpName, on_delete=models.CASCADE, related_name='part_set')
+
+    class Meta:
+        ordering = ['order']
+        unique_together = ['name', 'order']
+
+
+@reversion.register(follow=['part_set'])
+class RepositoryFamName(NameMixin):
+    """Corresponds to EAD3 famname as a child of repository."""
+    repository = models.ForeignKey(Repository, on_delete=models.CASCADE,
+                                   related_name='famname_set')
+
+
+@reversion.register()
+class RepositoryFamNamePart(PartMixin):
+    """Corresponds to EAD3 part as a child of famname and grandchild of
+    repository."""
+    name = models.ForeignKey(
+        RepositoryFamName, on_delete=models.CASCADE, related_name='part_set')
+
+    class Meta:
+        ordering = ['order']
+        unique_together = ['name', 'order']
+
+
+@reversion.register(follow=['part_set'])
+class RepositoryName(NameMixin):
+    """Corresponds to EAD3 name as a child of repository."""
+    repository = models.ForeignKey(Repository, on_delete=models.CASCADE,
+                                   related_name='name_set')
+
+
+@reversion.register()
+class RepositoryNamePart(PartMixin):
+    """Corresponds to EAD3 part as a child of repository."""
+    name = models.ForeignKey(
+        RepositoryName, on_delete=models.CASCADE, related_name='part_set')
+
+    class Meta:
+        ordering = ['order']
+        unique_together = ['name', 'order']
+
+
+@reversion.register(follow=['part_set'])
+class RepositoryPersName(NameMixin):
+    """Corresponds to EAD3 persname as a child of repository."""
+    repository = models.ForeignKey(Repository, on_delete=models.CASCADE,
+                                   related_name='persname_set')
+
+
+@reversion.register()
+class RepositoryPersNamePart(PartMixin):
+    """Corresponds to EAD3 part as a child of persname and grandchild of
+    repository."""
+    name = models.ForeignKey(
+        RepositoryPersName, on_delete=models.CASCADE, related_name='part_set')
+
+    class Meta:
+        ordering = ['order']
+        unique_together = ['name', 'order']
+
+
 @reversion.register()
 class Representation(CommonMixin, EncodingAnalogMixin, LangScriptMixin,
                      LinkMixin, LocalTypeMixin):
